#!/usr/bin/perl

# Copyright 2009, 2010, 2012 Kevin Ryde.

# This file is part of miscbits-el.
#
# miscbits-el is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3, or (at your option) any later
# version.
#
# miscbits-el is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with miscbits-el; see the file COPYING.  Failing that, go to
# <http://www.gnu.org/licenses/>.


# Usage: ./grep-emacs.pl
#

use strict;
use warnings;
use POSIX;
use Perl6::Slurp;
use Iterator::Simple qw(igrep);
use Iterator::Simple::Locate;

use lib::abs '.';
use MyStuff;
use MyUniqByInode;

my $verbose = 0;

my $it = Iterator::Simple::Locate->new (globs => [ '*.el' ]);
{
  my $uniq = MyUniqByInode->new;
  $it = igrep { $uniq->uniq($_) } $it;
}
$it = igrep { $_ =~ /site-lisp/ } $it;

while (my $filename = $it->next) {
  if ($verbose) { print "look at $filename\n"; }

  my $str;
  if (! eval { $str = Perl6::Slurp::slurp($filename); 1 }) {
    unless ($! == ENOENT) {
      print "Cannot read $filename: $!\n";
    }
    next;
  }

  # next unless ($str =~ /checkdoc-arguments-in-order-flag/m);

  # add-hook of lambda
  next unless $str =~ /^[^;]*(add-hook)
                       [^)\n]*\n?[^)\n]*
                       (\(function\s*)?
                       '?
                       (\#')?
                       \(lambda/mx;
  # turn-on-font-lock
  # (progn (require 'prolog) (customize-variable 'prolog-mode-hook))


  my ($line, $col) = MyStuff::pos_to_line_and_column ($str, $-[1]);
  print "$filename:$line:$col: found\n";
}

exit 0;
