;; Copyright 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;;-----------------------------------------------------------------------------

(let (lst)
  (mapatoms (lambda (symbol)
              (when (and (boundp symbol)
                         (not (string-match "-\\(function\\|mode\\|mode-map\\)\\'"
                                            (symbol-name symbol)))
                         (string-match "-indent\\(-offset\\)?"
                                       (symbol-name symbol))
                         (not (get symbol 'safe-local-variable)))
                (push symbol lst))))
  (setq lst (mapcar 'symbol-name lst))
  (setq lst (sort lst 'string-lessp))
  (dolist (str lst)
    (insert (format "%S\n" str))))
"change-log-indent-text"
"cperl-break-one-line-blocks-when-indent"
"cperl-fix-hanging-brace-when-indent"
"cperl-indent-region-fix-constructs"
"electric-indent-chars"
"facemenu-indentation-menu"
"fill-individual-varying-indent"
"filladapt-debug-indentation-extents"
"filladapt-debug-indentation-face-1"
"filladapt-debug-indentation-face-2"
"haskell-indent-after-keywords"
"haskell-indent-current-line-first-ident"
"haskell-indent-decision-table"
"haskell-indent-dont-hang"
"haskell-indent-inhibit-after-offset"
"haskell-indent-last-info"
"haskell-indent-literate-Bird-default-offset"
"haskell-indent-look-past-empty-line"
"haskell-indent-map"
"haskell-indent-off-side-keywords-re"
"haskell-indent-offset"
"haskell-indent-rhs-align-column"
"haskell-indent-start-keywords-re"
"haskell-indent-thenelse"
"locate-filename-indentation"
"lua-indent-level"
"lua-indent-whitespace"
"lua-indentation-modifier-regexp"
"mail-indentation-spaces"
"standard-indent"
"tab-always-indent"
"yaml-indent-offset"



(require 'yaml-mode)
(require 'haskell-indent)

;;-----------------------------------------------------------------------------



(put 'haskell-indent-offset 'safe-local-variable 'integerp)

(put 'yaml-indent-offset 'safe-local-variable 'integerp)
(shell-command "locate .yml|xargs egrep 'Local variables:'")

(put 'py-indent-offset 'safe-local-variable 'integerp)
(shell-command "locate .py|xargs egrep 'Local variables:'")



;;-----------------------------------------------------------------------------
(require 'sml-mode)

(put 'sml-indent-level   'safe-local-variable 'integerp)
(put 'sml-rightalign-and 'safe-local-variable 'booleanp)

(put 'sml-yacc-indent-action 'safe-local-variable 'integerp)
(put 'sml-yacc-indent-pipe   'safe-local-variable
     (lambda (value) (or (null value) (integerp value)))) ;; number or nil
(put 'sml-yacc-indent-term   'safe-local-variable
     (lambda (value) (or (null value) (integerp value)))) ;; number or nil
(shell-command "locate .sml|tee /dev/stderr |xargs egrep 'Local variables:'")

;;-----------------------------------------------------------------------------

;; /usr/share/emacs/site-lisp/lua-mode/lua-mode.el
(require 'lua)
(put 'lua-indent-level 'safe-local-variable 'integerp)
(shell-command "locate .lua|tee /dev/stderr |xargs egrep 'Local variables:'")

;;-----------------------------------------------------------------------------

(require 'slang-mode)
(put 'slang-indent-level 'safe-local-variable 'integerp)

(risky-local-variable-p 'slang-default-application)

(setq enable-local-variables t)
(setq enable-local-variables :all)
(find-file "~/bug/sml-mode/foo.sl")

;;-----------------------------------------------------------------------------
