;; Copyright 2009, 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (require 'checkdoc)

  (let ((count   0)
        (tempbuf "*t-checkdoc-keywords--temp*")
        (orig-checkdoc-y-or-n-p (symbol-function 'checkdoc-y-or-n-p))
        (orig-completing-read   (symbol-function 'completing-read)))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(checkdoc-keywords-comment))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((checkdoc-comment-style-hooks checkdoc-keywords-comment)))


    ;;-------------------------------------------------------------------------

    (unwind-protect
        (progn
          (fset 'checkdoc-y-or-n-p (lambda (question) t))
          (fset 'completing-read   (lambda (&rest args) "newkey"))

          (mapcar (lambda (elem)
                    (let* ((str  (nth 0 elem))
                           (want (nth 1 elem)))
                      (erase-buffer)
                      (insert str)
                      (checkdoc-keywords-comment)
                      (let ((got (buffer-string)))
                        (unless (equal want got)
                          (goto-char (point-max))
                          (insert (format "\n-------------------\nstr: %s\nwant: %s\n" str want))
                          (error "str %S got %S want %S" str got want)))
                      (setq count (1+ count))))

                  '((""
                     ";; Keywords: newkey\n")

                    (";; Keywords: nosuch"
                     ";; Keywords: nosuch, newkey")

                    (";; Keywords: nosuch,"
                     ";; Keywords: nosuch, newkey,")

                    (";; Keywords: nosuch,\t"
                     ";; Keywords: nosuch, newkey,\t")

                    (";; Keywords: lisp"
                     ";; Keywords: lisp")

                    ("\
;; Copyright 2010 Blah"
                     "\
;; Copyright 2010 Blah
;; Keywords: newkey
")

                    ("\
;; Copyright 2010 Blah

;; Version: 1
"
                     "\
;; Copyright 2010 Blah

;; Keywords: newkey
;; Version: 1
")

                    ("
;; Copyright 2010 Blah
;; Keywords: nosuch"
                     "
;; Copyright 2010 Blah
;; Keywords: nosuch, newkey"))))

      (fset 'checkdoc-y-or-n-p orig-checkdoc-y-or-n-p)
      (fset 'completing-read   orig-completing-read))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (unload-feature 'checkdoc-keywords t)


    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))
