;; Copyright 2006, 2007, 2009, 2010, 2011 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (interactive)
  (let ((count 0))

    (require 'cl)
    (require 'cl-loop-aplist)

    ;;-------------------------------------------------------------------------

    (let ((my-alist '((a . 1) (b . 2)))
          (my-plist '(a 1 b 2)))

      (mapcar (lambda (elem)
                (let* ((form  (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (condition-case err
                                 (eval form)
                               (error (error-message-string err)))))
                  (unless (equal got want)
                    (switch-to-buffer "*t-cl-loop-aplist temp*")
                    (erase-buffer)
                    (insert (format "got  %S\nwant %S\nform %s\nexpanded %s"
                                    got want
                                    (pp-to-string form)
                                    (pp-to-string (macroexpand form))))
                    (goto-char (point-min))

                    (error "got %S want %S\nform %S\nexpanded %S"
                           got want form
                           (macroexpand form))))
                (setq count (1+ count)))

              '(((loop for x being the alist-keys of my-alist
                       collect x)
                 (a b))
                ((loop for x being the alist-values of my-alist
                       collect x)
                 (1 2))

                ((loop for x being the plist-keys of my-plist
                       collect x)
                 (a b))
                ((loop for x being the plist-values of my-plist
                       collect x)
                 (1 2))


                ((loop for x being the alist-keys of my-alist
                       using (alist-values y)
                       collect (list x y))
                 ((a 1) (b 2)))
                ((loop for x being the alist-values of my-alist
                       using (alist-keys y)
                       collect (list x y))
                 ((1 a) (2 b)))

                ((loop for x being the plist-keys of my-plist
                       using (plist-values y)
                       collect (list x y))
                 ((a 1) (b 2)))
                ((loop for x being the plist-values of my-plist
                       using (plist-keys y)
                       collect (list x y))
                 ((1 a) (2 b)))


                ((loop for x being the alist-values of-ref my-alist
                       do (incf x)
                       finally return my-alist)
                 ((a . 2) (b . 3)))
                ((loop for x being the alist-keys of-ref my-alist
                       using-ref (alist-values y)
                       do (incf y)
                       finally return my-alist)
                 ((a . 3) (b . 4)))

                ((loop for x being the plist-values of-ref my-plist
                       do (incf x)
                       finally return my-plist)
                 (a 2 b 3))
                ((loop for x being the plist-keys of-ref my-plist
                       using-ref (plist-values y)
                       do (incf y)
                       finally return my-plist)
                 (a 3 b 4)))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (unload-feature 'cl-loop-aplist t)
      (mapcar (lambda (symbol)
                (when (eq (get symbol 'cl-loop-for-handler) 'cl-loop-aplist)
                  (error "cl-loop-for-handler not removed from %S" symbol)))
              '(alist-keys
                alist-values
                plist-keys
                plist-values))
      (setq count (1+ count)))

    ;; couple of cycles
    (require 'cl-loop-aplist)
    (unload-feature 'cl-loop-aplist t)
    (require 'cl-loop-aplist)
    (unload-feature 'cl-loop-aplist t)
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
