;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-compile-command ()
  (message "my-compile-command runs")
  "echo hi")

(defun my-run-test ()
  (interactive)
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (case-fold-search nil)
        (tempbuf "*t-compile-command-default--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'compile-command-default)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; risky-local-variable-p

    (if (not (fboundp 'risky-local-variable-p))
        (message "skip, no `risky-local-variable-p'")

      (mapcar (lambda (symbol)
                (unless (boundp symbol)
                  (error "oops, no variable: %S" symbol))
                (unless (risky-local-variable-p symbol (symbol-value symbol))
                  (error "%S not `risky-local-variable-p'" symbol))
                (setq count (1+ count)))
              '(compile-command-default-functions)))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'compile-command-default)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; compile-command-default--interpreter

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (let ((got (compile-command-default--interpreter)))
                  (unless (equal want got)
                    (error "compile-command-default--interpreter: %S\nwant %S\ngot  %S"
                           str want got)))
                (setq count (1+ count))))

            '(("blah"   nil)
              ("#!perl" "perl")
              ("#!/bin/sh" "/bin/sh")
              ("#!perl\n" "perl")
              ("#! perl\n" "perl")
              ("#!\t/usr/bin/perl" "/usr/bin/perl")
              ("#!/usr/bin/perl -w" "/usr/bin/perl -w")
              ("#!/usr/bin/perl -w -T" "/usr/bin/perl -w -T")
              ))


    ;;-------------------------------------------------------------------------
    ;; compile-command-default-perl-pl -- exercised

    (setq buffer-file-name "/tmp/t-compile-command-default--test.pl")
    (compile-command-default-perl-pl)
    (setq count (1+ count))

    (setq buffer-file-name "/tmp/t-compile-command-default--test.pl.gz")
    (compile-command-default-perl-pl)
    (setq count (1+ count))

    (setq buffer-file-name nil)


    ;;-------------------------------------------------------------------------
    ;; compile-command-default-perl-t-harness -- exercised

    (mapcar (lambda (elem)
              (let ((want (nth 0 elem))
                    (filename (nth 1 elem)))
                (setq buffer-file-name filename)
                (let ((got (not (not (compile-command-default-perl-t-harness)))))
                  (unless (equal got want)
                    (error "compile-command-default-perl-t-harness filename %S want %S got %S"
                           filename want got)))
                (setq buffer-file-name nil)
                (setq count (1+ count))))

            '((nil "/tmp/t-compile-command-default/test.pl")
              (t   "/tmp/t-compile-command-default/t/test.t")
              (t   "/tmp/t-compile-command-default/xt/test.t")
              (nil "/tmp/t-compile-command-default/devel/test.pl")
              (nil "/tmp/t-compile-command-default/devel/test.t")))

    ;;-------------------------------------------------------------------------
    ;; compile-command-default-perl-t-raw -- exercised

    (mapcar (lambda (elem)
              (let ((want (nth 0 elem))
                    (filename (nth 1 elem)))
                (setq buffer-file-name filename)
                (let ((got (not (not (compile-command-default-perl-t-raw)))))
                  (unless (equal got want)
                    (error "compile-command-default-perl-t-raw filename %S want %S got %S"
                           filename want got)))

                (setq buffer-file-name nil)
                (setq count (1+ count))))

            '((nil "/tmp/t-compile-command-default/test.pl")
              (t   "/tmp/t-compile-command-default/t/test.t")
              (t   "/tmp/t-compile-command-default/xt/test.t")
              (t   "/tmp/t-compile-command-default/devel/test.pl")
              (t   "/tmp/t-compile-command-default/devel/test.t")))


    ;;-------------------------------------------------------------------------
    ;; compile-command-default-functions -- unload cleanup

    (add-to-list 'load-path (expand-file-name "."))
    (require 'compile-command-default-otherpackage)

    (setq compile-command-default-functions
          '(compile-command-default-otherpackage-somefunc))
    (unload-feature 'compile-command-default-otherpackage t)

    (if (not (boundp 'unload-feature-special-hooks))
        (message "skip, no unload-feature-special-hooks")

      (when (memq 'compile-command-default-otherpackage-somefunc
                  compile-command-default-functions)
        (error "compile-command-default-otherpackage-somefunc not removed from -functions"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- defadvice

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'compile-command-default))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- hack-local-variables still works

    (require 'compile-command-default)
    (setq compile-command-default-functions
          '(my-compile-command))

    (unload-feature 'compile-command-default t)
    (erase-buffer)
    (setq fill-column 76)
    (setq buffer-file-name "t-compile-command-default.tmp")
    (kill-local-variable 'compile-command)
    (insert ";; Local " "variables:\n" ;; disguised from find-file
            ";; fill-column: 70\n"
            ";; End:\n")
    (hack-local-variables)
    (unless (equal fill-column 70)
      (error "hack-local-variables doesn't pick up fill-column after unload"))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'compile-command-default))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))

