;; Copyright 2009, 2010, 2011, 2013, 2015 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-in-first-line-p ()
  "Return non-nil if point is the first line of the buffer."
  (save-excursion
    (not (search-backward "\n" nil t))))

(defun my-run-test ()
  (require 'filladapt-pat)
  (let ((count 0)
        (tempbuf "*t-filladapt-pat--temp*"))
    (switch-to-buffer tempbuf)


    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(filladapt-pat-LocalWords
              filladapt-pat-dnl
              filladapt-pat-gp-comment
              filladapt-pat-no-numbered-bullets
              filladapt-pat-no-symbol-bullets
              filladapt-pat-no-postscript
              filladapt-pat-no-supercite
              filladapt-pat-no-citation->
              filladapt-pat-bullet-<li>
              filladapt-pat-bullet-<p>
              filladapt-pat-bullet-<!--
              filladapt-pat-bullet-pod
              filladapt-pat-version-bullet))

    ;;-------------------------------------------------------------------------
    ;; filladapt-pat-globally custom options

    (let ((options (get 'filladapt-pat-globally 'custom-options))
          commands)
      (mapatoms (lambda (symbol)
                  (and (string-match "\\`filladapt-pat-" (symbol-name symbol))
                       (commandp symbol)
                       (add-to-list 'commands symbol))))
      (setq options (sort (copy-sequence options)
                          (lambda (x y)
                            (string-lessp (symbol-name x)
                                          (symbol-name y)))))
      (setq commands (sort commands
                           (lambda (x y)
                             (string-lessp (symbol-name x)
                                           (symbol-name y)))))
      (unless (equal options commands)
        (insert "custom-options:\n")
        (pp options (current-buffer))
        (insert "commands:\n")
        (pp commands (current-buffer))
        (error "filladapt-pat-globally custom options %d but commands %d:\n%S\n%S"
               (length options)
               (length commands)
               options
               commands))
      (setq count (1+ count)))


    (if (not (locate-library "filladapt"))
        (message "skip, no filladapt.el")
      ;;-----------------------------------------------------------------------
      ;; filladapt-pat-no-* functions should remove things

      (require 'filladapt)
      (let (no-list)
        (mapatoms (lambda (symbol)
                    (and (string-match "\\`filladapt-pat-no-"
                                       (symbol-name symbol))
                         (commandp symbol)
                         (add-to-list 'no-list symbol))))
        (unless no-list
          (error "Oops, no filladapt-pat-no- functions"))
        (message "t-filladapt-pat.el: no-list funcs: %S" no-list)

        (dolist (func no-list)
          (erase-buffer)
          (fundamental-mode)
          (let ((old-ref  (copy-sequence filladapt-token-table))
                (old-copy (copy-sequence filladapt-token-table)))
            (funcall func)

            (unless (equal old-ref old-copy)
              (error "%S modified filladapt-token-table by side-effect" func))
            (when (equal old-copy filladapt-token-table)
              (error "%S didn't modify filladapt-token-table" func))

            (setq count (1+ count)))))


      ;;-----------------------------------------------------------------------
      ;; filladapt-pat-version-bullet

      (erase-buffer)
      (fundamental-mode)
      (filladapt-pat-version-bullet)
      (filladapt-mode 1)
      (setq fill-column 40)
      (insert "\
Version 1 - abc def ghi jkl mno pqr stu vwx yz abc def ghi jkl mno pqr stu vwx yz
Version 2 - abc def ghi jkl mno pqr stu vwx yz abc def ghi jkl mno pqr stu vwx yz
")
      (goto-char (point-min))
      (fill-paragraph nil)

      (let ((want "\
Version 1 - abc def ghi jkl mno pqr stu
            vwx yz abc def ghi jkl mno
            pqr stu vwx yz
Version 2 - abc def ghi jkl mno pqr stu vwx yz abc def ghi jkl mno pqr stu vwx yz
")
            (got (buffer-string)))
        (unless (equal want got)
          (error "filladapt-pat-version-bullet wrong")))
      (setq count (1+ count))


      ;;-----------------------------------------------------------------------
      ;; filladapt.el availability ...
      )

    (bury-buffer)
    (message "  %d tests" count)
    count))
