;; Copyright 2010, 2011, 2013, 2014, 2015, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  ;; (setq debug-on-error t)

  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-html-sizes.el temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(html-sizes-update))


    ;;-------------------------------------------------------------------------
    ;; load

    (require 'html-sizes)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; html-sizes-file-size-string

    (let ((filename "t-html-sizes.tmp"))
      (mapcar (lambda (elem)
                (let ((size     (nth 0 elem))
                      (decimals (nth 1 elem))
                      (unit     (nth 2 elem))
                      (suffix   (nth 3 elem))
                      (want     (nth 4 elem)))
                  (with-temp-buffer
                    (insert (make-string size ?x))
                    (write-region (point-min) (point-max) filename))

                  (let ((got (html-sizes-file-size-string filename decimals
                                                          unit suffix)))
                    (unless (equal got want)
                      (error "html-sizes-file-size-string %S %S %S %S\nwant %S\ngot  %S"
                             size decimals unit suffix
                             want got)))
                  (setq count (1+ count))))

              '(

                (0  0 "b" ""   "0")
                (0  2 "b" ""   "0")
                (0  2 "k" ""   "0.00")

                (2  3 "k" " kb"   "0.002 kb")
                (2  2 "k" " kb"   "0.01 kb")
                (15  2 "k" " kb"   "0.02 kb")

                (3000  0 "k" ""   "3")
                (3500  0 "k" ""   "4")
                (35000  0 "k" ""   "35")
                (35500  0 "k" ""   "36")

                (3000  1 "k" ""   "3.0")
                (3500  1 "k" ""   "3.5")
                (3550  1 "k" ""   "3.6")
                (3999  1 "k" ""   "4.0")

                (3000  2 "K" ""   "3.00")
                (3050  2 "k" ""   "3.05")
                (3055  2 "k" ""   "3.06")
                (3999  2 "k" ""   "4.00")

                (1000000  0 "M" ""   "1")
                (1500000  0 "M" ""   "2")
                (500000  1 "M" ""   "0.5")
                (10000  2 "M" ""   "0.01")

                ))
      (delete-file "t-html-sizes.tmp"))


    ;;-------------------------------------------------------------------------
    ;; html-sizes-image-size

    (when (html-sizes-image-size "nosuchfile")
      (error "html-sizes-image-size nosuchfile should be nil"))
    (setq count (1+ count))

    (when (html-sizes-image-size "nosuchfile.png")
      (error "html-sizes-image-size nosuchfile.png should be nil"))
    (setq count (1+ count))

    (when (html-sizes-image-size "nosuchfile.jpg")
      (error "html-sizes-image-size nosuchfile.jpg should be nil"))
    (setq count (1+ count))

    (let ((filename (if (boundp 'null-device)
                        null-device  ;; emacs21 up
                      "/dev/null"))) ;; emacs20
      (when (html-sizes-image-size filename)
        (error "html-sizes-image-size %S should be nil" filename))
      (setq count (1+ count)))

    ;; png size even on a tty
    (let ((got  (html-sizes-image-size "t-html-sizes--black260x5.png"))
          (want '(260 5)))
      (unless (equal got want)
        (error "html-sizes-image-size t-html-sizes--black260x5.png got %S want %S"
               got want))
      (setq count (1+ count)))


    ;;-----------------------------------------------------------------------
    ;; M-x html-sizes-update of <meta name="generator" ...>

    (setq html-sizes-generator "my generator string")
    (setq html-sizes-update-query nil)
    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (html-sizes-update)
                (let ((got (buffer-string)))
                  (unless (equal got want)
                    (error "want %S\ngot  %S" want got)))))
            '(("<head></head>"
               "<head><meta name=\"generator\" content=\"my generator string\">\n</head>")

              ;; if no </head> then no attempt to add meta
              (""
               "")))

    ;;-----------------------------------------------------------------------
    ;; M-x html-sizes-update of <img>

    (setq html-sizes-generator nil)
    (setq html-sizes-update-query nil)
    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str)
                (html-sizes-update)
                (let ((got (buffer-string)))
                  (unless (equal got want)
                    (error "want %S\ngot  %S" want got)))))
            '(("<img src=\"t-html-sizes.png\" width=99 height=7>"
               "<img src=\"t-html-sizes.png\" width=2 height=1>")

              ("<img src=\"t-html-sizes.png\" width=\"99\" height='7'>"
               "<img src=\"t-html-sizes.png\" width=\"2\" height='1'>")

              ("<img src=\"t-html-sizes.png\" width=2>"
               "<img src=\"t-html-sizes.png\" width=2 height=1>")

              ("<img src=\"t-html-sizes.png\">"
               "<img src=\"t-html-sizes.png\" width=2 height=1>")))

    ;;-----------------------------------------------------------------------
    ;; unload-feature

    (let ((feature 'html-sizes))
      (unload-feature feature t)
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'html-sizes)
      (kill-buffer nil))


    ;;-----------------------------------------------------------------------
    (bury-buffer)
    (message "  %d tests" count)
    count))
