;; Copyright 2006, 2007, 2009, 2010, 2016 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defconst my-msgfmt-available-p
  (condition-case nil
      (equal 0 (call-process "msgfmt"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"msgfmt\" program is available.")

(defconst my-msgunfmt-available-p
  (condition-case nil
      (equal 0 (call-process "msgunfmt"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"msgunfmt\" program is available.")

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(mo-mode))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'mo-mode)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    (if (not my-msgunfmt-available-p)
        (message "  skip, msgunfmt program not available")

      ;;-----------------------------------------------------------------------
      ;; decode working from a non-existent directory
      (let ((buffer (get-buffer-create "*t-mo-mode test*")))
        (switch-to-buffer buffer)
        (if (fboundp 'set-buffer-multibyte) ;; not in xemacs21
            (set-buffer-multibyte nil))
        (insert-file-contents-literally "t-mo-mode.mo")
        (setq default-directory "/no/such/dir/exists/")
        (mo-mode)
        (kill-buffer buffer)
        (setq count (1+ count)))

      ;;-----------------------------------------------------------------------
      ;; saving

      (if (not my-msgfmt-available-p)
          (message "  skip, msgfmt program not available")

        (let ((tempfile "t-mo-mode--temp.mo"))
          (copy-file "t-mo-mode.mo" tempfile)
          (unwind-protect
              (progn
                (find-file tempfile)
                (set-buffer-modified-p t)
                (save-buffer)

                (kill-buffer nil)
                (find-file tempfile)
                (kill-buffer nil)

                (setq count (1+ count)))

            (when (file-exists-p tempfile)
              (condition-case nil
                  (delete-file (file-newest-backup tempfile)) (error nil)))
            (condition-case nil
                (delete-file tempfile) (error nil))))

        ))

    ;;-------------------------------------------------------------------------
    ;; mo-mode--with-errorfile

    (if (not (fboundp 'mo-mode--with-errorfile))
        (message "skip, no mo-mode--with-errorfile after byte compile")

      (mo-mode--with-errorfile
       (unless (boundp 'errorfile)
         (error "mo-mode--with-errorfile doesn't bind `errorfile'"))
       (message "errorfile is %S" errorfile))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'mo-mode))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))


    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'mo-mode))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
