;; Copyright 2014, 2015 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  ;; (setq debug-on-error t)

  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (tempbuf "*t-oeis.el temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(oeis-browse-anum
              oeis-download
              oeis-ffap-enable
              oeis-grep
              oeis-search
              oeis-superseeker-mail))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'oeis)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; oeis-anum-fixup

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (oeis-anum-fixup str)))
                (unless (equal got want)
                  (error "oeis-anum-fixup on %S, want %S got %S" str want got))
                (setq count (1+ count))))

            '(("A123456" "A123456")
              ("A000000" "A000000")
              ("A000001" "A000001")
              ("A01"     "A000001")
              ("A1234567"  "A1234567")
              ("A01234567" "A1234567")

              ("b123456.txt" "A123456")
              ("b012345.txt" "A012345")
              ("a12345.txt" "A012345")
              ("b1234567.txt" "A1234567")
              ("b001.txt" "A000001")
              ))

    ;;-------------------------------------------------------------------------
    ;; oeis-number-list-at-point

    (mapcar (lambda (elem)
              (let ((str       (nth 0 elem))
                    (pos       (nth 1 elem))
                    (separator (nth 2 elem))
                    (want      (nth 3 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (oeis-number-list-at-point separator)))
                  (unless (equal got want)
                    (error "oeis-number-list-at-point\nwant %S\ngot  %S" want got)))
                (setq count (1+ count))))

            '(("1, 2, 3,\n4, 5, 6" 0 nil   "1,2,3,4,5,6")
              ("1 2 3" 0 nil               "1,2,3")
              ("1,2,3" 0 " "               "1 2 3")
              ("1,+2,-3" 0 " "             "1 2 -3")

              ;; FIXME: not sure if 1.234 should be made to return nil.
              ;; Calling for decimal-digits before number-list might be
              ;; enough.
              ;; ("1.234" 0 nil               nil)

              (".1234" 0 nil               nil)
              ("1234." 0 nil               "1234")
              ("blah" 0 nil                nil)
              ("A000001" 0 nil             nil)
              ("A000001" 3 nil             nil)

              ("1.5 123 456" 0 nil   "1")
              ("1.5 123 456" 3 nil   "123,456")))

    ;;-------------------------------------------------------------------------
    ;; oeis-decimal-digits-at-point

    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (pos  (nth 1 elem))
                    (want (nth 2 elem)))
                (erase-buffer)
                (insert str)
                (goto-char (+ (point-min) pos))
                (let ((got (oeis-decimal-digits-at-point)))
                  (unless (equal got want)
                    (error "want %S\ngot  %S" want got)))
                (setq count (1+ count))))

            '(;; sans leading +/-
              ("-0.12345" 5    "1,2,3,4,5")
              ("+0.12345" 5    "1,2,3,4,5")
              
              ("blah 12.345" 5  "1,2,3,4,5")

              ;; sans leading 0 digits
              ("0.12345" 5    "1,2,3,4,5")
              ("009.12345" 5  "9,1,2,3,4,5")
              ("0.00012345" 5 "1,2,3,4,5")

              ;; non-matches
              ("blah" 0 nil)
              ("A000001" 0 nil)
              ("A000001" 3 nil)))

    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'oeis)
      (kill-buffer nil))

    ;;-----------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'oeis))
        (oeis-ffap-enable)
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))

        ;; check nothing left in ffap-alist
        (mapcar (lambda (pair)
                  (let* ((value (cdr pair))
                         (name (if (symbolp value)
                                   (symbol-name value)
                                 "")))
                    (when (string-match "\\`oeis-" name)
                      (error "unload-feature left entry in ffap-alist"))))
                ffap-alist)
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'oeis))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      (oeis-ffap-enable)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; ffap-string-at-point -- still works when unloaded

    (erase-buffer)
    (let ((got  (ffap-string-at-point))
          (want ""))
      (unless (equal got want)
        (error "ffap-string-at-point in empty buffer after unload got: %S"
               got))
      (setq count (1+ count)))

    ;;-----------------------------------------------------------------------
    (kill-buffer tempbuf)
    (message "  %d tests" count)
    count))
