;; Copyright 2010 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)

  (let ((count 0)
        (case-fold-search t)
        ;; (tempbuf "*t-perl-env-substitute.el temp*")
        )
    ;; (switch-to-buffer tempbuf)
    ;; (erase-buffer)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(perl-env-substitute-enable
              perl-env-substitute-ffap-braces))

    (mapcar (lambda (func)
              (unless (commandp func)
                (error "`%S' not interactive" func))
              (setq count (1+ count)))
            '(perl-env-substitute-enable
              perl-env-substitute-ffap-braces))


    ;;-------------------------------------------------------------------------
    ;; load

    (require 'perl-env-substitute)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; perl-env-substitute-ENV

    (mapcar (lambda (elem)
              (let* ((str  (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (perl-env-substitute-ENV str)))
                (unless (equal got want)
                  (error "str %S\ngot  %S\nwant %S" str got want)))
              (setq count (1+ count)))

            '(("$FOO"        "$FOO")
              ("${FOO}"      "${FOO}")
              ("$ENV{FOO}"   "${FOO}")
              ("$Env{FOO}"   "$Env{FOO}")
              ("$ENV{'FOO'}" "${FOO}")
              ("$ENV{'FOO'}$ENV{'BAR'}" "${FOO}${BAR}")
              ("$ENV{'FOO'}xyz$ENV{'BAR'}" "${FOO}xyz${BAR}")
              ("$ENV{FOO"    "${FOO")
              ))


    ;;-------------------------------------------------------------------------
    ;; perl-env-substitute-ffap-braces

    (perl-env-substitute-ffap-braces)
    (require 'cl) ;; for copy-tree
    (let ((old (copy-tree ffap-string-at-point-mode-alist)))
      (perl-env-substitute-ffap-braces)
      (let ((new ffap-string-at-point-mode-alist))
        (unless (equal old new)
          (error "perl-env-substitute-ffap-braces changes ffap-string-at-point-mode-alist on second run\n%S\n%S" old new))))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- defadvice cleanup, emacs22 up

    (require 'perl-env-substitute)

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'perl-env-substitute))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'perl-env-substitute))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    ;; (bury-buffer)
    (message "  %d tests" count)
    count))
