;; Copyright 2007, 2008, 2009, 2010 Kevin Ryde.   -*- coding: latin-1 -*-

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(add-to-list 'load-path (expand-file-name ".."))

(defconst my-gzip-available-p
  (condition-case nil
      (equal 0 (call-process "gzip"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"gzip\" program is available.")

(defun my-iota (n)
  "Return a list of numbers (0 1 2 ... N-1)."
  (let ((i 0)
        ret)
    (while (< i n)
      (setq ret (cons i ret))
      (setq i (1+ i)))
    (nreverse ret)))

(defun my-props (str)
  (mapcar (lambda (i)
            (text-properties-at i str))
          (my-iota (length str))))

(defun my-propertize (str &rest props)
  "Same as `propertize', which is not available in xemacs."
  (setq str (copy-sequence str))
  (add-text-properties 0 (length str) props str)
  str)

;;   (defun my-props-equal (p1 p2)
;;     (equal (sort p1 'my-prop-lessp)
;;            (sort p2 'my-prop-lessp)))
;;
;;   (defun my-prop-lessp (p1 p2)
;;     (string-lessp (symbol-name p1) (symbol-name p2)))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0)
        (middle-dot "") ;; latin-1
        (tempbuf "*t-tty-format--temp*"))
    (switch-to-buffer tempbuf)

    ;;-------------------------------------------------------------------------
    ;; autoloads

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(ansi-format-decode
              ansi-format-encode
              backspace-overstrike-decode
              backspace-overstrike-encode
              tty-format-guess))

    (mapcar (lambda (symbol)
              (unless (memq t
                            (mapcar (lambda (elem)
                                      (and (eq symbol (car-safe elem))
                                           (progn
                                             (mapcar (lambda (func)
                                                       (unless (fboundp func)
                                                         (error "`%S' not autoloaded" func)))
                                                     (list (nth 3 elem)
                                                           (nth 4 elem)))
                                             t)))
                                    format-alist))
                (error "`%S' not autoloaded in `format-alist'" symbol))
              (setq count (1+ count)))
            '(ansi-colors
              backspace-overstrike))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            (if (boundp 'find-file-hook)
                '((find-file-hook tty-format-guess))   ;; emacs22
              '((find-file-hooks  tty-format-guess)))) ;; xemacs21


    ;;-------------------------------------------------------------------------
    ;; `backspace-overstrike-decode'

    (mapcar (lambda (elem)
              (let* ((str        (nth 0 elem))
                     (want       (nth 1 elem))
                     (want-props (my-props want))
                     (case-fold-search t))
                (erase-buffer)
                (insert str)
                (backspace-overstrike-decode (point-min) (point-max))
                (let* ((got       (buffer-string))
                       (got-props (my-props got)))
                  (unless (and (string-equal want got)
                               (equal want-props got-props))
                    (goto-char (point-max))
                    (insert (format "\ncount: %d\ngot: %s\n\nwant: %s\n%s\norig: %s\n"
                                    count got-props want want-props str))
                    (error "wrong"))))
              (setq count (1+ count)))

            (list (list "+\b+\bo\bo" (my-propertize middle-dot 'face 'bold))
                  (list "x\bx"    (my-propertize "x" 'face 'bold))
                  (list "_\bx"    (my-propertize "x" 'face 'underline))
                  (list "x\b_"    (my-propertize "x" 'face 'underline))
                  (list "_\bx\bx" (my-propertize "x" 'face '(bold underline)))
                  (list "x\b_\bx" (my-propertize "x" 'face '(bold underline)))
                  (list "x\bx\b_" (my-propertize "x" 'face '(bold underline)))
                  (list "+\bo"    middle-dot)
                  (list "_\b+\bo"    (my-propertize middle-dot 'face 'underline))
                  (list "_\bab\bb"   (concat (my-propertize "a" 'face 'underline)
                                             (my-propertize "b" 'face 'bold)))
                  (list " _\ba b\bb " (concat " "
                                              (my-propertize "a" 'face 'underline)
                                              " "
                                              (my-propertize "b" 'face 'bold)
                                              " "))
                  (list "|\b="     "|\b=")
                  (list "|\b=x\bx" (concat "|\b="
                                           (my-propertize "x" 'face 'bold)))
                  (list "x\bx|\b=" (concat (my-propertize "x" 'face 'bold)
                                           "|\b="))
                  (list "\b"     "\b")
                  (list "\b\b\b" "\b\b\b")
                  (list "a\bA"   "a\bA")  ;; not on mixed case
                  (list  "z\bx\bx" (my-propertize "z\bx" 'face 'bold))

                  (list  (my-propertize "x\bx" 'face 'underline)
                         (my-propertize "x" 'face '(underline bold)))

                  (list  (my-propertize "x\bx" 'face 'bold)
                         (my-propertize "x" 'face 'bold))

                  (list  (my-propertize "_\bx\bx" 'face '(bold underline))
                         (my-propertize "x" 'face '(bold underline)))

                  ))

    ;;-------------------------------------------------------------------------
    ;; `tty-format-add-faces'

    (progn
      (erase-buffer)
      (insert "x")
      (tty-format-add-faces '(bold) (point-min) (point-max))
      (tty-format-add-faces '(bold) (point-min) (point-max))
      (let* ((got        (buffer-string))
             (got-props  (my-props got))
             (want-props '((face bold))))
        (unless (equal want-props got-props)
          (goto-char (point-max))
          (insert (format "\ngot: %s\n\nwant: %s\n"
                          got-props want-props))
          (error "tty-format-add-faces wrong")))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; tty-format-guess

    (let ((file-list '("t-tty-format.txt"
                       "t-tty-format-ansi.txt")))

      ;; exercise jka-compr-byte-compiler-base-file-name bit with a .gz file
      ;; too, when possible
      (if my-gzip-available-p
          (progn
            (auto-compression-mode 1)
            (setq file-list (append file-list '("t-tty-format.txt.gz"))))
        (message "skip t-tty-format.txt.gz as gzip program not available"))

      (mapcar (lambda (filename)
                (find-file filename)

                (tty-format-guess)

                ;; last "z" char should be boldened
                (let* ((got        (buffer-substring (- (point-max) 2)
                                                     (- (point-max) 1)))
                       (got-props  (my-props got))
                       (want-props '((face bold))))
                  (unless (equal want-props got-props)
                    (switch-to-buffer tempbuf)
                    (erase-buffer)
                    (insert (format "\ngot: %s\n\nwant: %s\n"
                                    got-props want-props))
                    (error "file %S wrong" filename)))

                (kill-buffer nil)
                (setq count (1+ count)))

              file-list))

    ;;-------------------------------------------------------------------------

    (bury-buffer)
    (message "  %d tests" count)
    count))

